#****************************************************************************
#**
#**  File     :  /cdimage/units/XSB2401/XSB2401_script.lua
#**  Author(s):  John Comes, David Tomandl, Matt Vainio
#**
#**  Summary  :  Seraphim Tactical Missile Launcher Script
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local SStructureUnit = import('/lua/seraphimunits.lua').SStructureUnit
local SIFExperimentalStrategicMissile = import('/lua/seraphimweapons.lua').SIFExperimentalStrategicMissile
local EffectUtil = import('/lua/EffectUtilities.lua')
local EffectTemplate = import('/lua/EffectTemplates.lua')

local Buff = import('/lua/sim/Buff.lua')
local EffectTemplate = import('/lua/EffectTemplates.lua')
local Entity = import('/lua/sim/Entity.lua').Entity
local AIUtils = import('/lua/AI/aiutilities.lua')


XSB2401 = Class(SStructureUnit) {


    Weapons = {

    },

	    AmbientEffects = 'ST2PowerAmbient',
	    ShieldEffects = {
        '/effects/emitters/seraphim_regenerative_aura_01_emit.bp',
    	},


    # Copy of XSL0001_script's RegenBuffThread with slight alterations (Enhancement and RegenField)
    RegenBuffThread = function(self)
        while not self:IsDead() do
            # Get friendly units in the area (including self)
            local units = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.ALLUNITS, self:GetPosition(), self:GetBlueprint().RegenAura.RegenRadius)
           # Give them a 5 second regen buff
            for _,unit in units do
	        if unit != self then
                   Buff.ApplyBuff(unit, 'XPAura')
		end
            end
            #Wait 5 seconds
            WaitSeconds(5)
        end
    end,




    OnStopBeingBuilt = function(self)
 
        #SStructureUnit.OnStopBeingBuilt(self,builder,layer)
        local bp = self:GetBlueprint()
        self.Trash:Add(CreateAnimator(self):PlayAnim(bp.Display.AnimationOpen))
        self:ForkThread(self.PlayArmSounds)


        if not self.ShieldEffectsBag then
            self.ShieldEffectsBag = {}
        end
        self.ShieldEffectsBag = {}
        local army =  self:GetArmy()
        if self.AmbientEffects then
            for k, v in EffectTemplate[self.AmbientEffects] do
				CreateAttachedEmitter(self, 'XSB2401', army, v):ScaleEmitter(1.2)
            end
        end
               
        local bp = self:GetBlueprint().RegenAura

        if not Buffs['XPAura'] then
            BuffBlueprint {
                Name = 'XPAura',
                DisplayName = 'XPAura',
                BuffType = 'ACUREGENERATEBONUS',
                Stacks = 'ALWAYS',
                Duration = 5,
                Affects = {
                    Regen = {
                        Add = 200,
                        Mult = 1.0,   #bp.RegenPerSecond or 0.1,
                        #Ceil = bp.RegenCeiling,
                        #Floor = bp.RegenFloor,
                    },
                    #MaxHealth = {
                    #    Add = 0,
                    #    Mult = bp.MaxHealthFactor or 1.0,
                    #    DoNoFill = true,
                    #},                        
                },
            }
        end

        table.insert( self.ShieldEffectsBag, CreateAttachedEmitter( self, 'XSB2401', self:GetArmy(), '/effects/emitters/seraphim_regenerative_aura_01_emit.bp' ) )
        self.RegenThreadHandle = self:ForkThread(self.RegenBuffThread)

    end,




    
    PlayBuildEffects = function(self)
        if not self.MissileEffect then 
            self.MissileEffect = {} 
        end
        local effectBones = self:GetBlueprint().General.BuildBones.BuildEffectBones
        for bidx, bone in effectBones do 
            for k, v in EffectTemplate.SJammerCrystalAmbient do
                if not self.MissileEffect[bidx] then
                    self.MissileEffect[bidx] = {}
                end
                self.MissileEffect[bidx][k] = CreateAttachedEmitter(self,bone, self:GetArmy(), v)
            end
        end
        self:PlayUnitSound('Construct')
        self:PlayUnitAmbientSound('ConstructLoop')                      
    end,
    
    StopBuildEffects = function(self)
        local effectBones = self:GetBlueprint().General.BuildBones.BuildEffectBones
        for bidx, bone in effectBones do 
            for k, v in EffectTemplate.SJammerCrystalAmbient do
                self.MissileEffect[bidx][k]:Destroy()
            end
        end
        self:StopUnitAmbientSound('ConstructLoop')
        self:PlayUnitSound('ConstructStop')             
    end,
    

    

    
    
        

}
TypeClass = XSB2401
